#ifndef DCMSIMULATOR_H
#define DCMSIMULATOR_H

#include "DAQDataFormats/FEBSimulator.h"	/* Front End Board Simulator */
#include "DAQDataFormats/RawMicroSlice.h"	/* Microslice Format */

namespace daqdataformats {

#define DCM_DATA_BUFFER_SIZE 65536 /* 64k data buffer */

class DCMSimulator {

 public:

  DCMSimulator();
  ~DCMSimulator();
  DCMSimulator(uint32_t diblock_number, uint32_t dcm_number);
  DCMSimulator(uint32_t FEBMask);
  DCMSimulator(uint32_t FEBMask, float probability);

  bool init();
  bool initFEB(int FEBnumber);
  bool initFEBList();

  void setFEBMask(uint32_t FEBMask);
  void setHitProbability(float probability);

  RawMicroSlice* generateMicroSlice();

  void printMicroSlice();

 private:

  uint32_t dcmID;		/* Local DCM identifier 1-12    */
  uint32_t diblockID;	/* Diblock identifier   1-30ish */
  uint32_t clock;		/* The dcm timing clock */

  uint32_t febMask;		/* Mask for the FEB links */
  
  FEBSimulator FrontEndBoards[64];  /* 64 Copies of Front End Boards */
  FEBSimulator* ActiveFEBList[64];  /* Pointers to the active Front End Boards */

  float hit_probability;

  uint32_t databuffer[DCM_DATA_BUFFER_SIZE];  /* Main Data buffer */
  uint32_t* nextWord;

  RawMicroSlice LastMicroSlice;	   /* Last Generated MicroSlice */
};

} /* END NAMESPACE daqdataformats */
#endif /* FEBSIMULATOR_H */
