#ifndef RAWEVENTHEADER_H
#define RAWEVENTHEADER_H
#include "DAQDataFormats/RawEventHeaderV1.h"
/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEventHeader
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE raweventhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        static const version_t RAWEVENTHEADER_LATEST_VERSION = LAST_CLASS_VERSION;
    }

    class RawEventHeader: public PREVIOUS_VERSION_CLASS_NAME{
        public:

            RawEventHeader(const version_t version = CURRENT_FIELDS_SUB_NAMESPACE::RAWEVENTHEADER_LATEST_VERSION); //!< Constructor
            RawEventHeader(const RawEventHeader& copy_in);	    //!< Copyin Constructor
            RawEventHeader(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);  //!< Constructor with data intialization
            ~RawEventHeader();                                    //!< Destructor

            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN(MarkerLeft      , uint32_t)
            IMPLEMENT_GET_MAIN(MarkerRight     , uint32_t)
            IMPLEMENT_GET_MAIN(Version         , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(DataLoggerID    , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunNumber       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunType         , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ConfigIDX       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Partition       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Subrun          , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(EventNumberLo   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(EventNumberHi   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(EventType       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(EventSize       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(DataBlockNumber , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerFilled   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TailFilled      , uint32_t)

            IMPLEMENT_GET_AND_SET_MAIN(MonteCarloFlag, bool);
            IMPLEMENT_IS_AND_SET_MAIN      (CRCCalculationUsed);
            IMPLEMENT_IS_AND_SET_MAIN      (EventIncomplete);

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addEventSize          , ,bool       , const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(advanceEventNumber    , ,bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(advanceDataBlockNumber, ,bool)

            IMPLEMENT_GET_AND_SET_MAIN(EventNumber, uint64_t)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(CURRENT_CLASS)

        protected:

            bool             setFunctionPointers();

            IMPLEMENT_SET_MAIN(MarkerLeft      , uint32_t)
            IMPLEMENT_SET_MAIN(MarkerRight     , uint32_t)
            IMPLEMENT_SET_MAIN(Version         , uint32_t)


    };


} // END NAMESPACE daqdataformats





#endif /* RAWEVENTHEADER_H */
