#ifndef DAQRAWTRIGGERTIMEMARKERV0_H
#define DAQRAWTRIGGERTIMEMARKERV0_H

#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerTimingMarker
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggertimingmarker
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace VERSION_NAMESPACE {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {

                static const uint32_t TRIGGERTIMINGMARKER_SIZE = 4;
            
                //! Enumeration of the Masks for the Trigger Timing Marker
                enum TriggerTimeMarkerMASKS{
                    MASK_TriggerStartLow_baseClock  = 0xFFFFFFFF,
                    MASK_TriggerStartHigh_baseClock = 0x00FFFFFF,
                    MASK_Version                    = 0xFF000000,
                    MASK_TriggerStartLow_833kHz     = 0xFFFFFFFF,
                    MASK_TriggerStartHigh_833kHz    = 0x00FFFFFF
                };
            
                //! Enumeration of the Words for the Trigger Timing Marker
                enum TriggerTimeMarkerWORDS{
                    WORD_TriggerStartLow_baseClock  = 0,
                    WORD_TriggerStartHigh_baseClock = 1,
                    WORD_Version                    = 1,
                    WORD_TriggerStartLow_833kHz     = 2,
                    WORD_TriggerStartHigh_833kHz    = 3
                };
                DEFINE_SHIFT_FROM_MASK(TriggerStartLow_baseClock)
                DEFINE_SHIFT_FROM_MASK(TriggerStartHigh_baseClock)
                DEFINE_SHIFT_FROM_MASK(Version)
                DEFINE_SHIFT_FROM_MASK(TriggerStartLow_833kHz)
                DEFINE_SHIFT_FROM_MASK(TriggerStartHigh_833kHz)
      }


  //! Class that contains the timing information relating to a given trigger
  /*!
    The RawTriggerTimingMarkerV0 contains the start indicator for data collection.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerTimingMarker: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            RawTriggerTimingMarker (const version_t); ///< The default constructor
            RawTriggerTimingMarker (const RawTriggerTimingMarker&);
            ~RawTriggerTimingMarker(); ///< Destructor

            void     init      ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print     (std::ostream& os=std::cout) const;
            uint32_t sizeofdata() const { return CURRENT_FIELDS_SUB_NAMESPACE::TRIGGERTIMINGMARKER_SIZE;}
            /// Nanoslice of version 0 is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            IMPLEMENT_FIRST_GET_AND_SET(Version                   , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(TriggerStartLow_baseClock , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerStartHigh_baseClock, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerStartLow_833kHz    , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerStartHigh_833kHz   , uint32_t)

	    INIT_FIRST(getTriggerStart_baseClock, const, uint64_t, )
            INIT_FIRST(setTriggerStart_baseClock,      , bool    , const uint64_t)
	    INIT_FIRST(getTriggerStart_833kHz   , const, uint64_t, )
            INIT_FIRST(setTriggerStart_833kHz   ,      , bool    , const uint64_t)
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERTIMEMARKERV0_H */
