#ifndef _DBUTILS_DAQCONFIG_BNEVBRUNCONFIG_H
#define _DBUTILS_DAQCONFIG_BNEVBRUNCONFIG_H

#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <map>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching of BufferNodeEVB run configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2013/12/11 14:14:40 $
 */

class BNEVBRunConfig {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    BNEVBRunConfig(const std::vector<int64_t>& subsystemConfigIdList);

    BNEVBRunConfig(const SubsystemNameSpec& nameSpec);

    BNEVBRunConfig(const GlobalNameSpec& nameSpec);

    BNEVBRunConfig(const int64_t& globalConfigId);

    virtual ~BNEVBRunConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}

    double getPoolSizeSeconds() {return _poolSizeSeconds;}

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList);

    bool _fetchSystemData(const std::string& tableDefinitionFileName,
                          const int64_t& subsystemConfigId);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    double _poolSizeSeconds;
};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
