/* 
 * File:   DAQChannelMapBaseOLD.cpp
 * Author: denis
 * 
 * Created on February 10, 2012, 10:17 AM
 */

#include "DAQChannelMap/DAQChannelMapBaseOLD.h"
#include "NovaDAQConventions/DAQConventions.h"

#include <iostream>

using namespace daqchannelmap;

// Standard enumeration
#define PUT_ENUM(INPUT) CHAN_IDX_0_1                 \
                        INPUT##_NEARDET,             \
                        INPUT##_FARDET,              \
                        INPUT##_NDOS,                \
			INPUT##_TESTBEAM_2Block_1x1, \
			INPUT##_TESTBEAM_3Block_1x1, \
                        INPUT##_TESTBEAM_2Block_2x2, \
                        INPUT##_TESTBEAM_3Block_2x2, \
                        INPUT##_IPND                


const unsigned int DAQChannelMapBaseOLD::NUM_XCHAN_PLANE[] = { PUT_ENUM(NUM_CHAN_X_PLANE) };
const unsigned int DAQChannelMapBaseOLD::NUM_YCHAN_PLANE[] = { PUT_ENUM(NUM_CHAN_Y_PLANE)};
const unsigned int DAQChannelMapBaseOLD::NUM_DIBLOCK_DET[] = { PUT_ENUM(NUM_DIBLOCK) };
const unsigned int DAQChannelMapBaseOLD::NUM_DCMS_DIBLOCK[]= { PUT_ENUM(NUM_DCMS_DIBLOCK) };
const unsigned int DAQChannelMapBaseOLD::NUM_X_DCMS[]      = { PUT_ENUM(NUM_X_DCMS) };
const unsigned int DAQChannelMapBaseOLD::NUM_Y_DCMS[]      = { PUT_ENUM(NUM_Y_DCMS) };


const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_IPND[]    = {CHAN_IDX_0_1
									 AB_DIBLOCK,AB_DIBLOCK,
									 TERM_CHAR
                                                                         };

const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_NEARDET[] = {CHAN_IDX_0_1
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
                                                                         MC_ND_DIBLOCK, // Muon catcher for NearDet
									 TERM_CHAR
                                                                         };

const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_NDOS[]    = {CHAN_IDX_0_1
									 AB_DIBLOCK,AB_DIBLOCK,AA_DIBLOCK,
									 MC_DIBLOCK, // Muon catcher for NDOS (actually it's just incomplete B-block)
									 TERM_CHAR
                                                                         };

const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_FARDET[]  = {CHAN_IDX_0_1
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
									 AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
									 TERM_CHAR
                                                                         };


const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_TESTBEAM_2Block_1x1[] = {CHAN_IDX_0_1
										     AFULLAFULL_DIBLOCK,
										     TERM_CHAR
                                                                                     };
const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_TESTBEAM_3Block_1x1[] = {CHAN_IDX_0_1
										     AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
										     TERM_CHAR
                                                                                     };
const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_TESTBEAM_2Block_2x2[] = {CHAN_IDX_0_1
										     AFULLAFULL_DIBLOCK,
										     TERM_CHAR
                                                                                     };
const unsigned int DAQChannelMapBaseOLD::DetectorDiBlockConfig_TESTBEAM_3Block_2x2[] = {CHAN_IDX_0_1
										     AFULLAFULL_DIBLOCK,AFULLAFULL_DIBLOCK,
										     TERM_CHAR
                                                                                     };

// Mapping of APD, index is the Pixel Channel, result is the module cell
const unsigned int DAQChannelMapBaseOLD::apd_pixel2cell_v0[] = {25,24,27,26,29,28,31,30,
						             17,16,19,18,21,20,22,23,
						             15,14,12,13,10,11, 8, 9,
						             6, 7, 4, 5, 2, 3, 0, 1
                                                             };
// Back Mapping of APD, index is the module cell, result is the APD channel
const unsigned int DAQChannelMapBaseOLD::apd_cell2pixel_v0[] = {30,31,28,29,26,27,24,25,
							     22,23,20,21,18,19,17,16,
							     9, 8,11,10,13,12,14,15,
							     1, 0, 3, 2, 5, 4, 7, 6};

//------------------------------------------------------------------------------
DAQChannelMapBaseOLD::DAQChannelMapBaseOLD():
     _planesInDiblockDetectorArray(NULL)
    ,_planesInBlockDetectorArray(NULL)
    ,_planesInBlockDetectorSize(NULL)
    ,_planesInDiblockDetectorSize(NULL)
    ,_planesBeforeDiblockDetectorArray(NULL)
    ,_planesBeforeBlockDetectorArray(NULL)
    ,_BlockOrderInDiblockArray(NULL)
    ,_numberOfTypePlanesInBlockArray(NULL)
    ,_numberOfTypePlanesInDiBlockArray(NULL)
    ,_numberOfXDCMsInDiblockDetectorArray(NULL)
    ,_numberOfYDCMsInDiblockDetectorArray(NULL)
    ,_planeType_planeNumberArray(NULL)

{

    initDetectorDiBlockConfig();

    initApdPixel2PixelMap();

    initNumberOfCellsPerDCMInDiblock();

    initBlockOrderInDiblock();

    initPlanesInBlockID();

    initNumberOfTypePlanesInBlock();

    initPlanesInDiblockID();
    
    initStartPlaneTypeInBlock();

    initPlanesInBlockAndDiblockDetector();

    initNumberOfDCMsInDiblockDetector();

    initPlaneTypePlaneNumber();
}

//------------------------------------------------------------------------------
DAQChannelMapBaseOLD::~DAQChannelMapBaseOLD() {
    if(_BlockOrderInDiblockArray)         free(_BlockOrderInDiblockArray);
    if(_numberOfTypePlanesInBlockArray)   free(_numberOfTypePlanesInBlockArray);
    if(_numberOfTypePlanesInDiBlockArray) free(_numberOfTypePlanesInDiBlockArray);
    if(_numberOfXDCMsInDiblockDetectorArray) free(_numberOfXDCMsInDiblockDetectorArray);
    if(_numberOfYDCMsInDiblockDetectorArray) free(_numberOfYDCMsInDiblockDetectorArray);

    if(_planesInDiblockDetectorArray) free(_planesInDiblockDetectorArray);
    if(_planesInBlockDetectorArray) free(_planesInBlockDetectorArray);
    if(_planesBeforeDiblockDetectorArray) free(_planesBeforeDiblockDetectorArray);
    if(_planesBeforeBlockDetectorArray) free(_planesBeforeBlockDetectorArray);
    if(_planeType_planeNumberArray) free(_planeType_planeNumberArray);

    if(_planesInBlockDetectorSize) free(_planesInBlockDetectorSize);
    if(_planesInDiblockDetectorSize) free(_planesInDiblockDetectorSize);

    if(DetectorDiBlockConfigArray) free(DetectorDiBlockConfigArray);
}



//------------------------------------------------------------------------------
/// Initialization methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initApdPixel2PixelMap(){

       /// First, need to resize the maps
       _apdPixel2CellMap.resize(32);
       _cell2ApdPixelMap.resize(32);

       const unsigned int*  apd_pixel2cell_pointer = apd_pixel2cell_v0;

       /// First, construct the map of apdPixel2CellMap
       for(uint32_t iapd_pixel = 0; iapd_pixel < 32; ++iapd_pixel){
           const pixel_t ipixel = (pixel_t) (apd_pixel2cell_pointer[iapd_pixel]);
           _apdPixel2CellMap[iapd_pixel] = ipixel;
           _cell2ApdPixelMap[ipixel]     = iapd_pixel;
       }

       _apdPixel2CellMapArray = &(_apdPixel2CellMap[0]);
       _cell2ApdPixelMapArray = &(_cell2ApdPixelMap[0]);

       return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initNumberOfCellsPerDCMInDiblock(){

    _numberOfCellsPerDCMInDiblock.resize(MAX_DIBLOCK_TYPE+1);
    _numberOfCellsPerDCMInDiblock[AB_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[BA_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[AA_DIBLOCK]         = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[AFULLAFULL_DIBLOCK] = NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[MC_DIBLOCK]         = 2*NUM_CELLS_PER_DCM;
    _numberOfCellsPerDCMInDiblock[MC_ND_DIBLOCK]      = 2*NUM_CELLS_PER_DCM;

    _numberOfCellsPerDCMInDiblockArray = &(_numberOfCellsPerDCMInDiblock[0]);

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitNumberOfCellsPerDCMInDiblock(){

    /// Needs _numberOfCellsPerDCMInDiblock initialized
    if( _numberOfCellsPerDCMInDiblock.size() < MAX_BLOCK_TYPE+1 && !initNumberOfCellsPerDCMInDiblock() ) {
            std::cerr<<"initNumberOfCellsPerDCMInDiblock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfCellsPerDCMInDiblock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initBlockOrderInDiblock(){

    for(uint32_t idiblock_type = 0; idiblock_type <= MAX_DIBLOCK_TYPE; idiblock_type++){
        std::vector <uint32_t> temp_vector;
        temp_vector.resize(2);
        _BlockOrderInDiblock.push_back(temp_vector);
    }/// end of loop over diblock types

    _BlockOrderInDiblock[AB_DIBLOCK][0]         = A_BLOCK;
    _BlockOrderInDiblock[AB_DIBLOCK][1]         = B_BLOCK;
    _BlockOrderInDiblock[BA_DIBLOCK][0]         = B_BLOCK;
    _BlockOrderInDiblock[BA_DIBLOCK][1]         = A_BLOCK;
    _BlockOrderInDiblock[AA_DIBLOCK][0]         = A_BLOCK;
    _BlockOrderInDiblock[AA_DIBLOCK][1]         = A_BLOCK;
    _BlockOrderInDiblock[AFULLAFULL_DIBLOCK][0] = AFULL_BLOCK;
    _BlockOrderInDiblock[AFULLAFULL_DIBLOCK][1] = AFULL_BLOCK;
    _BlockOrderInDiblock[MC_DIBLOCK][0]         = M_BLOCK;
    _BlockOrderInDiblock[MC_DIBLOCK][1]         = M_BLOCK;
    _BlockOrderInDiblock[MC_ND_DIBLOCK][0]      = MC_ND_BLOCK;
    _BlockOrderInDiblock[MC_ND_DIBLOCK][1]      = MC_ND_BLOCK;

    allocateArray(_BlockOrderInDiblockArray, _BlockOrderInDiblock);

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitBlockOrderInDiblock(){

    /// Needs BlockOrderInDiblock initialized
    if( _BlockOrderInDiblock.size() < MAX_DIBLOCK_TYPE+1 && !initBlockOrderInDiblock() ) {
            std::cerr<<"BlockOrderInDiblock can not be initialized\n";
            exit(1);
    }// end of check whether BlockOrderInDiblock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initPlanesInBlockID(){

    _planesInBlockType.resize(MAX_BLOCK_TYPE+1);
    _planesInBlockType[A_BLOCK]     = NUM_XZ_PLANES_A_BLOCK     + NUM_YZ_PLANES_A_BLOCK;
    _planesInBlockType[B_BLOCK]     = NUM_XZ_PLANES_B_BLOCK     + NUM_YZ_PLANES_B_BLOCK;
    _planesInBlockType[AFULL_BLOCK] = NUM_XZ_PLANES_AFULL_BLOCK + NUM_YZ_PLANES_AFULL_BLOCK;
    _planesInBlockType[M_BLOCK]     = NUM_XZ_PLANES_MC_BLOCK    + NUM_YZ_PLANES_MC_BLOCK;
    _planesInBlockType[MC_ND_BLOCK] = NUM_XZ_PLANES_MC_ND_BLOCK + NUM_YZ_PLANES_MC_ND_BLOCK;

    _planesInBlockTypeArray = &(_planesInBlockType[0]);

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitPlanesInBlockID(){

    /// Needs _planesInBlockType initialized
    if( _planesInBlockType.size() < MAX_BLOCK_TYPE+1 && !initPlanesInBlockID() ) {
            std::cerr<<"_planesInBlockType can not be initialized\n";
            exit(1);
    }// end of check whether _planesInBlockType is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initNumberOfTypePlanesInBlock(){

        isinitPlanesInBlockID();

        numberOfPlanesInBlock_t number_of_yz_planes_in_block;
        numberOfPlanesInBlock_t number_of_xz_planes_in_block;
        number_of_yz_planes_in_block.resize(MAX_BLOCK_TYPE+1);
        number_of_xz_planes_in_block.resize(MAX_BLOCK_TYPE+1);

        number_of_yz_planes_in_block[A_BLOCK]     = NUM_YZ_PLANES_A_BLOCK;     ///< Number of vertical planes. A-Block starts with horizontal plane
        number_of_yz_planes_in_block[B_BLOCK]     = NUM_YZ_PLANES_B_BLOCK;     ///< B-block starts with vertical plane
        number_of_yz_planes_in_block[AFULL_BLOCK] = NUM_YZ_PLANES_AFULL_BLOCK; ///< AFULL-block starts with horizontal plane
        number_of_yz_planes_in_block[M_BLOCK]     = NUM_YZ_PLANES_MC_BLOCK;    ///< NDOS Muon catcher starts with vertical plane
        number_of_yz_planes_in_block[MC_ND_BLOCK] = NUM_YZ_PLANES_MC_ND_BLOCK; ///< NearDet Muon catcher starts with horizontal plane (I think...)

        /// Calculating number of XZ planes for each Block Type
        for(block_t iblock_type = MIN_BLOCK_TYPE; iblock_type <= MAX_BLOCK_TYPE; iblock_type++){
            const DetBlock_TYPE block_type           = (DetBlock_TYPE)iblock_type;
            number_of_xz_planes_in_block[block_type] = getNumberOfPlanesInBlock(block_type) - number_of_yz_planes_in_block[block_type];
        }// end of loop over blocks calculating number of XZ planes for each Block Type

        _numberOfTypePlanesInBlock.resize(2);
        _numberOfTypePlanesInBlock[XZ_PLANE-1] = number_of_xz_planes_in_block;
        _numberOfTypePlanesInBlock[YZ_PLANE-1] = number_of_yz_planes_in_block;

        allocateArray(_numberOfTypePlanesInBlockArray, _numberOfTypePlanesInBlock);

        return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitNumberOfTypePlanesInBlock(){

    /// Needs _numberOfTypePlanesInBlock initialized
    if( _numberOfTypePlanesInBlock.size() < 2 && !initNumberOfTypePlanesInBlock() ) {
            std::cerr<<"_numberOfTypePlanesInBlock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfTypePlanesInBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initPlanesInDiblockID(){


    /// Needs PlanesInBlockID initialized
    isinitPlanesInBlockID();
    /// Needs BlockOrderInDiblock initialized
    isinitBlockOrderInDiblock();
    /// Needs NumberOfTypePlanesInBlock initialized
    isinitNumberOfTypePlanesInBlock();

    /// Filling Number of planes in Diblock and each of its views
    _planesInDiblockType.resize(MAX_DIBLOCK_TYPE+1);

    _numberOfTypePlanesInDiBlock.resize(2);
    _numberOfTypePlanesInDiBlock[XZ_PLANE-1].resize(MAX_DIBLOCK_TYPE+1);
    _numberOfTypePlanesInDiBlock[YZ_PLANE-1].resize(MAX_DIBLOCK_TYPE+1);
    for(uint32_t idiblock_type = MIN_DIBLOCK_TYPE; idiblock_type <= MAX_DIBLOCK_TYPE; idiblock_type++){
        const DetBlock_TYPE block0 = (DetBlock_TYPE)_BlockOrderInDiblock[idiblock_type][0];
        const DetBlock_TYPE block1 = (DetBlock_TYPE)_BlockOrderInDiblock[idiblock_type][1];

        _planesInDiblockType[idiblock_type] = getNumberOfPlanesInBlock(block0) + getNumberOfPlanesInBlock(block1);
        _numberOfTypePlanesInDiBlock[XZ_PLANE-1][(DiBlock_TYPE)idiblock_type] = getNumberOfPlaneTypeInBlock(block0, XZ_PLANE) + getNumberOfPlaneTypeInBlock(block1, XZ_PLANE);
        _numberOfTypePlanesInDiBlock[YZ_PLANE-1][(DiBlock_TYPE)idiblock_type] = getNumberOfPlaneTypeInBlock(block0, YZ_PLANE) + getNumberOfPlaneTypeInBlock(block1, YZ_PLANE);
    }// end of loop over diblocks filling Number of planes in Diblock and each of its views


    allocateArray(_numberOfTypePlanesInDiBlockArray, _numberOfTypePlanesInDiBlock);


    _planesInDiblockTypeArray = &_planesInDiblockType[0];


    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitPlanesInDiblockID(){

    /// Needs _numberOfTypePlanesInDiBlock initialized
    if( _numberOfTypePlanesInDiBlock.size() < 2 && !initPlanesInDiblockID() ) {
            std::cerr<<"_numberOfTypePlanesInDiBlock can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfTypePlanesInDiBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initStartPlaneTypeInBlock(){

    _startPlaneTypeInBlock.resize(MAX_BLOCK_TYPE+1);
    _startPlaneTypeInBlock[A_BLOCK]     = XZ_PLANE; /// A-block starts with vertical plane
    _startPlaneTypeInBlock[B_BLOCK]     = YZ_PLANE; /// B-block starts with horizontal plane
    _startPlaneTypeInBlock[AFULL_BLOCK] = YZ_PLANE; /// AFULL-block starts with horizontal plane
    _startPlaneTypeInBlock[M_BLOCK]     = YZ_PLANE; /// M-block starts with horizontal plane
    _startPlaneTypeInBlock[MC_ND_BLOCK] = YZ_PLANE; /// MC-ND-block starts with a horizontal module


    _startPlaneTypeInBlockArray = &(_startPlaneTypeInBlock[0]);

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitStartPlaneTypeInBlock(){

    /// Needs _startPlaneTypeInBlock initialized
    if( _startPlaneTypeInBlock.size() < MAX_BLOCK_TYPE+1 && !initStartPlaneTypeInBlock() ) {
            std::cerr<<"_startPlaneTypeInBlock can not be initialized\n";
            exit(1);
    }// end of check whether _startPlaneTypeInBlock is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initPlanesInBlockAndDiblockDetector(){

    /// Needs PlanesInBlockID initialized
    isinitPlanesInBlockID();
    /// Needs BlockOrderInDiblock initialized
    isinitBlockOrderInDiblock();

    /// Fill Number of Planes in Diblock
    for(uint32_t idet = 0; idet < novadaq::cnv::kNDetector; idet++){

        std::vector <uint32_t> number_of_planes_in_diblock_for_current_detector;
        std::vector <uint32_t> number_of_planes_in_block_for_current_detector;

        std::vector <uint32_t> number_of_planes_before_diblock_for_current_detector;
        std::vector <uint32_t> number_of_planes_before_block_for_current_detector;

        /// Fill stuff only for legal detectors, otherwise leave vectors empty
        if(idet > novadaq::cnv::kUNKNOWN_DET && idet<novadaq::cnv::kNDetector){


            // we index from 1 on diblocks and blocks, so need to push 0
            for(int i=0; i<BLOCK_OFFSET; ++i){
                number_of_planes_in_block_for_current_detector.push_back(0);
                number_of_planes_before_block_for_current_detector.push_back(0);
            }
            for(int i=0; i<DIBLOCK_OFFSET; ++i){
                number_of_planes_in_diblock_for_current_detector.push_back(0);
                number_of_planes_before_diblock_for_current_detector.push_back(0);
            }


            int current_n_planes = 0;
            /// Loop over all diblock numbers in the detector
            for(diblock_t idiblock = DIBLOCK_OFFSET; idiblock < MAX_DIBLOCK+DIBLOCK_OFFSET; idiblock++){

                const DiBlock_TYPE diblock_type = getDiBlockType(idet, idiblock);

                if(diblock_type == (int)TERM_CHAR) break;

                const uint32_t nplanes_in_diblock = getNumberOfPlanesInDiblock(diblock_type);

                /// Fill planes for diblock
                number_of_planes_in_diblock_for_current_detector.push_back(nplanes_in_diblock);

                number_of_planes_before_diblock_for_current_detector.push_back(
                    number_of_planes_before_diblock_for_current_detector[number_of_planes_before_diblock_for_current_detector.size() - 1]
                   +nplanes_in_diblock
                );

                /// Blocks in the current diblock
                const DetBlock_TYPE block0             = getBlockType(diblock_type, (uint32_t) 0);
                const DetBlock_TYPE block1             = getBlockType(diblock_type, (uint32_t) 1);
                const int           nplanes_in_block_0 = getNumberOfPlanesInBlock(block0);
                const int           nplanes_in_block_1 = getNumberOfPlanesInBlock(block1);


                /// Fill planes for two blocks of the current diblock
                number_of_planes_in_block_for_current_detector.push_back(nplanes_in_block_0);
                number_of_planes_in_block_for_current_detector.push_back(nplanes_in_block_1);

                current_n_planes += nplanes_in_block_0;
                number_of_planes_before_block_for_current_detector.push_back(current_n_planes);
                current_n_planes += nplanes_in_block_1;
                number_of_planes_before_block_for_current_detector.push_back(current_n_planes);
            }// end of loop over diblocks
        }// end of if detector is legal


        _planesInDiblockDetector.push_back(number_of_planes_in_diblock_for_current_detector);
        _planesInBlockDetector.push_back(number_of_planes_in_block_for_current_detector);
    }

    allocateArray(_planesInDiblockDetectorArray, _planesInDiblockDetector);
    allocateArray(_planesInBlockDetectorArray, _planesInBlockDetector);

    allocateSize(_planesInDiblockDetectorSize, _planesInDiblockDetector);
    allocateSize(_planesInBlockDetectorSize, _planesInBlockDetector);


    _planesBeforeBlockDetector.resize(novadaq::cnv::kNDetector);
    _planesBeforeDiblockDetector.resize(novadaq::cnv::kNDetector);

    /// Fill Number of Planes in Diblock
    for(uint32_t idet = 0; idet < novadaq::cnv::kNDetector; idet++){

        /// Fill stuff only for legal detectors, otherwise leave vectors empty
        if(idet > novadaq::cnv::kUNKNOWN_DET && idet<novadaq::cnv::kNDetector){

            const uint32_t num_diblocks = getTotalNumberOfDiblocks(idet) + DIBLOCK_OFFSET;
            const uint32_t num_blocks   = getTotalNumberOfBlocks(idet)   + BLOCK_OFFSET;

            std::vector<uint32_t> nplanes_before_diblock;
            std::vector<uint32_t> nplanes_before_block;


            // we index from 1 on diblocks and blocks, so need to push 0
            for(int i=0; i<BLOCK_OFFSET; ++i){
                nplanes_before_block.push_back(0);
                nplanes_before_block.push_back(0);
            }
            for(int i=0; i<DIBLOCK_OFFSET; ++i){
                nplanes_before_diblock.push_back(0);
                nplanes_before_diblock.push_back(0);
            }

            uint32_t planes = 0;
            /// Count all planes before this Diblock. Add them to the offline plane number
            for(diblock_t idiblock = 1 + DIBLOCK_OFFSET; idiblock < num_diblocks; idiblock++){
                planes += getNumberOfPlanesInDiblock(idet, idiblock - DIBLOCK_OFFSET);
                nplanes_before_diblock.push_back(planes);
            }// end of loop over diblocks before the current one

            _planesBeforeDiblockDetector[idet] = nplanes_before_diblock;

            planes = 0;
            /// Count all planes before this Block. Add them to the offline plane number
            for(block_t iblock = 1 + BLOCK_OFFSET; iblock < num_blocks; iblock++){
                planes += getNumberOfPlanesInBlock(idet, iblock - BLOCK_OFFSET);
                nplanes_before_block.push_back(planes);
            }// end of loop over blocks before the current one

            _planesBeforeBlockDetector[idet] = nplanes_before_block;
            }
    
    }// end of loop over detectors

    allocateArray(_planesBeforeDiblockDetectorArray, _planesBeforeDiblockDetector);
    allocateArray(_planesBeforeBlockDetectorArray, _planesBeforeBlockDetector);

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitPlanesInBlockAndDiblockDetector(){

    /// Needs _planesInBlockDetector initialized
  if( _planesInBlockDetector.size() < novadaq::cnv::kNDetector && !initPlanesInBlockAndDiblockDetector() ) {
            std::cerr<<"_planesInBlockDetector can not be initialized\n";
            exit(1);
    }// end of check whether _planesInBlockDetector is initialized

    return true;
}


//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initNumberOfDCMsInDiblockDetector(){

    /// Initialize vectors
    for(uint32_t detector_id = 0; detector_id <= novadaq::cnv::kNDetector; ++detector_id){
        std::vector<uint32_t> tmp_vector;
        tmp_vector.resize(MAX_DIBLOCK_TYPE+1);

        _numberOfXDCMsInDiblockDetector.push_back(tmp_vector);
        _numberOfYDCMsInDiblockDetector.push_back(tmp_vector);
    }// end of initializing vectors


    for(uint32_t  detector = 0; detector <= novadaq::cnv::kNDetector; detector++){
        for(diblock_t idiblock = MIN_DIBLOCK_TYPE; idiblock <= MAX_DIBLOCK_TYPE; ++idiblock){
        _numberOfXDCMsInDiblockDetector[detector][idiblock] = NUM_X_DCMS[detector];
        _numberOfYDCMsInDiblockDetector[detector][idiblock] = NUM_Y_DCMS[detector];
        }
    }// end of filling number of DCMs


    /// Exceptions are the muon catchers, for which number of DCMs is 1 in each view
    _numberOfXDCMsInDiblockDetector[novadaq::cnv::kNDOS][MC_DIBLOCK] = 1;
    _numberOfYDCMsInDiblockDetector[novadaq::cnv::kNDOS][MC_DIBLOCK] = 1;
    _numberOfXDCMsInDiblockDetector[novadaq::cnv::kNEARDET][MC_ND_DIBLOCK] = 1;
    _numberOfYDCMsInDiblockDetector[novadaq::cnv::kNEARDET][MC_ND_DIBLOCK] = 1;


    allocateArray(_numberOfXDCMsInDiblockDetectorArray, _numberOfXDCMsInDiblockDetector);
    allocateArray(_numberOfYDCMsInDiblockDetectorArray, _numberOfYDCMsInDiblockDetector);

    return true;
}// end of initNumberOfXDCMsInDiblockDetector

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::isinitNumberOfDCMsInDiblockDetector(){

    /// Needs _numberOfXDCMsInDiblockDetector initialized
    if( _numberOfXDCMsInDiblockDetector.size() < novadaq::cnv::kNDetector && !initNumberOfDCMsInDiblockDetector() ) {
            std::cerr<<"_numberOfXDCMsInDiblockDetector can not be initialized\n";
            exit(1);
    }// end of check whether _numberOfXDCMsInDiblockDetector is initialized

    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initPlaneTypePlaneNumber(){

    for(uint32_t idet = 0; idet < novadaq::cnv::kNDetector; ++idet){
        std::vector <uint32_t> planeType_plane;

        _totalNumberOfPlanes.push_back(0);
        
        /// Fill stuff only for legal detectors, otherwise leave vectors empty
        if(idet > novadaq::cnv::kUNKNOWN_DET && idet<novadaq::cnv::kNDetector){

            const uint32_t total_number_of_diblocks = getTotalNumberOfDiblocks(idet) + DIBLOCK_OFFSET;

            for(diblock_t idiblock = DIBLOCK_OFFSET; idiblock < total_number_of_diblocks; ++idiblock){
                _totalNumberOfPlanes[idet] = _totalNumberOfPlanes[idet] + getNumberOfPlanesInDiblock(idet, idiblock);
            }

            planeType_plane.resize(_totalNumberOfPlanes[idet]);


            for(offline_plane_t iplane = 0; iplane < _totalNumberOfPlanes[idet]; ++iplane){

                /// current block number
                const diblock_t     diblock_number                = getDiblockNumber(idet, iplane);
                const block_t       block_number                  = getBlockNumber(idet, iplane);

                const DiBlock_TYPE  diblock_type                  = getDiBlockType(idet, diblock_number);

                const uint32_t      local_plane_in_block          = getLocalPlaneInBlock(idet, iplane);

                const uint32_t      local_block_number_in_diblock = (block_number - BLOCK_OFFSET) % 2;
                
                const DetBlock_TYPE block_type                    = getBlockType(diblock_type, local_block_number_in_diblock);

                const DetPlane_TYPE starting_plane                = getStartPlaneTypeInBlock(block_type);

                if(local_plane_in_block % 2 == 0)
                    planeType_plane[iplane] = starting_plane;
                else
                    planeType_plane[iplane] = getOtherPlaneType(starting_plane);
                
            }// end of loop over planes
            
        }
        _planeType_planeNumber.push_back(planeType_plane);

    }// end of loop over detectors


    allocateArray(_planeType_planeNumberArray, _planeType_planeNumber);
    

    //print();
    return true;

}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::initDetectorDiBlockConfig(){

    /// Loop over all possible detector IDs
    for(int i=0; i< 100; ++i){
        if(i == novadaq::cnv::kUNKNOWN_DET) DetectorDiBlockConfigVector.push_back(getVector(0,0));
        if(i == novadaq::cnv::kNEARDET)     DetectorDiBlockConfigVector.push_back(getVector(DetectorDiBlockConfig_NEARDET));
        if(i == novadaq::cnv::kFARDET)      DetectorDiBlockConfigVector.push_back(getVector(DetectorDiBlockConfig_FARDET));
        if(i == novadaq::cnv::kNDOS)        DetectorDiBlockConfigVector.push_back(getVector(DetectorDiBlockConfig_NDOS));
        if(i == novadaq::cnv::kNDSBTEST)    DetectorDiBlockConfigVector.push_back(getVector(DetectorDiBlockConfig_NEARDET));
        if(i == novadaq::cnv::kTESTBEAM)    DetectorDiBlockConfigVector.push_back(getVector(DetectorDiBlockConfig_TESTBEAM_2Block_1x1));
    }

    allocateArray(DetectorDiBlockConfigArray, DetectorDiBlockConfigVector);

    return true;
}


//------------------------------------------------------------------------------
void DAQChannelMapBaseOLD::printDiBlockConfigVector(){
    std::cout<<"Printing DiBlockConfigVector\n";


    int ndetectors = DetectorDiBlockConfigVector.size();
    for(int idet = 0; idet < ndetectors; ++idet){
        int ndiblocks = DetectorDiBlockConfigVector[idet].size();

        std::cout<<"Detector "<<idet<<"\n";
        std::cout<<"N diblocks = "<<ndiblocks<<"\n";
    }

}

//------------------------------------------------------------------------------
std::vector<uint32_t> DAQChannelMapBaseOLD::getVector(const unsigned int* inn, const unsigned int maxsize){

    std::vector<uint32_t> out;

    if(maxsize == 0)
        out.push_back(0);

    for(uint32_t i = 0; i < maxsize; ++i){
        out.push_back(inn[i]);
        if(inn[i] == TERM_CHAR)break;
    }
    
    return out;
}



//------------------------------------------------------------------------------
bool    DAQChannelMapBaseOLD::print() const{

    std::cout<<__PRETTY_FUNCTION__<<"\n";

    /// loop over detectors
    for(uint32_t idet = 0; idet < novadaq::cnv::kNDetector; idet++){

        /// working with legal detector IDs only
        if(idet > novadaq::cnv::kUNKNOWN_DET && idet<novadaq::cnv::kNDetector){

            std::cout<<"Detector ID"<<idet<<"\n";

            const uint32_t number_of_planes = getTotalNumberOfPlanes(idet);

            for(offline_plane_t iplane = 0; iplane < number_of_planes; ++iplane){
                print(idet, iplane);
                std::cout<<"\n";
            }
        }
    }// end of loop over detectors


    return true;

}
//------------------------------------------------------------------------------
bool    DAQChannelMapBaseOLD::print(const int idet, const offline_plane_t iplane) const {


                /// current block number
                const diblock_t     diblock_number                = getDiblockNumber(idet, iplane);
                const block_t       block_number                  = getBlockNumber(idet, iplane);

                const DiBlock_TYPE  diblock_type                  = getDiBlockType(idet, diblock_number);

                const uint32_t      local_block_number_in_diblock = (block_number - BLOCK_OFFSET) % 2;

                const DetBlock_TYPE block_type                    = getBlockType(diblock_type, local_block_number_in_diblock);


                std::cout<<"Plane="<<iplane
                         <<" Diblock type="<<diblock_type
                         <<" Block type="<<block_type
                         <<" local_block_number_in_diblock="<<local_block_number_in_diblock
                         <<" Plane type="<<getPlaneType(idet, iplane)
                         <<" NumberOfChannels="<<getNumberOfChannelsInPlane(idet, iplane);

                return true;
}



//------------------------------------------------------------------------------
/// Get minimum detector ID
int           DAQChannelMapBaseOLD::getDetectorIDMinimum() const {return novadaq::cnv::kNEARDET;}
//------------------------------------------------------------------------------
/// Get maximum detector ID
int           DAQChannelMapBaseOLD::getDetectorIDMaximum() const {return novadaq::cnv::kNDetector - 1;}

//------------------------------------------------------------------------------
DetPlane_TYPE   DAQChannelMapBaseOLD::getPlaneType(const int detId, const offline_plane_t plane_0) const{
    return (DetPlane_TYPE) _planeType_planeNumberArray[detId][plane_0];
}

//------------------------------------------------------------------------------
pixel_t       DAQChannelMapBaseOLD::APDPixel2Pixel(apd_pixel_t apd_pixel) const  {return _apdPixel2CellMapArray[apd_pixel];}
apd_pixel_t   DAQChannelMapBaseOLD::pixel2APDPixel(pixel_t     pixel)     const  {return _cell2ApdPixelMapArray[pixel];}


DiBlock_TYPE  DAQChannelMapBaseOLD::getDiBlockType(int detector, diblock_t diblock_number) const {return (DiBlock_TYPE)DetectorDiBlockConfigArray[detector][diblock_number];}


uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesInBlock      (int detector, block_t   block  ) const {return _planesInBlockDetectorArray[detector][block];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesInDiblock    (int detector, diblock_t diblock) const {return _planesInDiblockDetectorArray[detector][diblock];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesBeforeBlock  (int detector, block_t   block  ) const {return _planesBeforeBlockDetectorArray[detector][block];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesBeforeDiblock(int detector, diblock_t diblock) const {return _planesBeforeDiblockDetectorArray[detector][diblock];}

uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesInBlock  (DetBlock_TYPE block_type  ) const {return _planesInBlockTypeArray[block_type];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlanesInDiblock(DiBlock_TYPE  diblock_type) const {return _planesInDiblockTypeArray[diblock_type];}

uint32_t      DAQChannelMapBaseOLD::getBlockNumber   (int detector, offline_plane_t plane_0) const {return getBlockOrDiblock(detector, plane_0, _planesInBlockDetectorSize[detector]  ,(const uint**)_planesInBlockDetectorArray  , BLOCK_OFFSET);}
uint32_t      DAQChannelMapBaseOLD::getDiblockNumber (int detector, offline_plane_t plane_0) const {return getBlockOrDiblock(detector, plane_0, _planesInDiblockDetectorSize[detector],(const uint**)_planesInDiblockDetectorArray, DIBLOCK_OFFSET);}

//------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getDiblockNumber(const block_t block_number) const
{
  // WARNING:  Assumption of 2 blocks per diblock.
  return DIBLOCK_OFFSET + (block_number - BLOCK_OFFSET)/2;
}

//------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getLocalBlockInDiblock (const block_t block_number) const
{
    return ODD(block_number-BLOCK_OFFSET);
}

uint32_t      DAQChannelMapBaseOLD::getNumberOfXDCM(int detector, DiBlock_TYPE  diblock_type) const{return _numberOfXDCMsInDiblockDetectorArray[detector][diblock_type];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfYDCM(int detector, DiBlock_TYPE  diblock_type) const{return _numberOfYDCMsInDiblockDetectorArray[detector][diblock_type];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfDCM (int detector, DiBlock_TYPE  diblock_type) const {return getNumberOfXDCM(detector, diblock_type) + getNumberOfYDCM(detector, diblock_type);}
uint32_t      DAQChannelMapBaseOLD::getNumberOfChannelsInXPlane(int detector) const{return NUM_XCHAN_PLANE[detector];}
uint32_t      DAQChannelMapBaseOLD::getNumberOfChannelsInYPlane(int detector) const{return NUM_YCHAN_PLANE[detector];}

//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getNumberOfChannelsInPlane(const int detId, const DetPlane_TYPE plane_type) const{

    if(plane_type == XZ_PLANE)
        return getNumberOfChannelsInXPlane(detId);

    return getNumberOfChannelsInYPlane(detId);
}
//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getNumberOfChannelsInPlane(const int detId, const offline_plane_t plane_0) const{
    return getNumberOfChannelsInPlane(detId, getPlaneType(detId, plane_0));
}
//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlaneTypeInBlock  (DetBlock_TYPE block_type  , DetPlane_TYPE plane_type) const {
    return _numberOfTypePlanesInBlockArray[plane_type-1][block_type];
}
//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getNumberOfPlaneTypeInDiblock(DiBlock_TYPE  diblock_type, DetPlane_TYPE plane_type) const {
    return _numberOfTypePlanesInDiBlockArray[plane_type-1][diblock_type];
}

//---------------------------------------------------------------------------------------
DetBlock_TYPE DAQChannelMapBaseOLD::getBlockType(DiBlock_TYPE diblock_type, uint32_t local_block_number_in_diblock) const{return (DetBlock_TYPE)_BlockOrderInDiblockArray[diblock_type][local_block_number_in_diblock];}

//---------------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapBaseOLD::getStartPlaneTypeInBlock(DetBlock_TYPE block_type) const {
    return _startPlaneTypeInBlockArray[block_type];
}

//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getNumberOfCellsPerDCM(DiBlock_TYPE diblock_type) const{ return _numberOfCellsPerDCMInDiblockArray[diblock_type];}
//---------------------------------------------------------------------------------------
// WARNING:  Assumption of 2 blocks per diblock.
uint32_t      DAQChannelMapBaseOLD::getTotalNumberOfBlocks  (int detector) const { return 2 * getTotalNumberOfDiblocks(detector);}
//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getTotalNumberOfDiblocks(int detector) const { return _planesInDiblockDetector[detector].size() - DIBLOCK_OFFSET;}
//---------------------------------------------------------------------------------------
uint32_t      DAQChannelMapBaseOLD::getTotalNumberOfPlanes  (const int detId) const { return _totalNumberOfPlanes[detId];}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapBaseOLD::getBlockOrDiblock(int                                    detector
                                             ,offline_plane_t                        plane_0
                                             ,const numberOfPlanesInBlockDetector_t& planes_in_block_or_diblock
                                             ,const BlockNumerationOffset            block_enum_offset
                                             ) const
{

  CHECKERROR( plane_0);

  offline_plane_t planescount                      = 0;
  uint32_t        max_number_of_blocks_or_diblocks = planes_in_block_or_diblock[detector].size();
  uint32_t        iblock_or_diblock                = block_enum_offset; //we index from 0 or 1 on diblocks and blocks
  for(; iblock_or_diblock < max_number_of_blocks_or_diblocks; ++iblock_or_diblock) {
      planescount += planes_in_block_or_diblock[detector][iblock_or_diblock];
      if(plane_0 < planescount) return iblock_or_diblock;
  }

  return iblock_or_diblock;
}

//------------------------------------------------------------------------------
uint32_t DAQChannelMapBaseOLD::getBlockOrDiblock(int                         detector
                                             ,offline_plane_t             plane_0
                                             ,const uint32_t              max_number_of_blocks_or_diblocks
                                             ,const uint**                planes_in_block_or_diblock
                                             ,const BlockNumerationOffset block_enum_offset
                                             ) const
{

  CHECKERROR( plane_0);

  offline_plane_t planescount        = 0;
  uint32_t        iblock_or_diblock  = block_enum_offset; //we index from 0 or 1 on diblocks and blocks
  for(; iblock_or_diblock < max_number_of_blocks_or_diblocks; ++iblock_or_diblock) {
      planescount += planes_in_block_or_diblock[detector][iblock_or_diblock];
      if(plane_0 < planescount) return iblock_or_diblock;
  }

  return iblock_or_diblock;
}


//------------------------------------------------------------------------------
// Returns local plane number for the diblock from offline channel
offline_plane_t DAQChannelMapBaseOLD::getLocalPlaneInDiblock(int detector, offline_plane_t plane_0) const
{
  return getLocalPlaneInBlockOrDiblock(detector, plane_0, _planesInDiblockDetectorSize[detector], (const uint32_t**)_planesInDiblockDetectorArray, DIBLOCK_OFFSET);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapBaseOLD::getLocalPlaneInBlock(int detector, offline_plane_t plane_0) const
{
  return getLocalPlaneInBlockOrDiblock(detector, plane_0, _planesInBlockDetectorSize[detector], (const uint32_t**)_planesInBlockDetectorArray, BLOCK_OFFSET);
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapBaseOLD::getLocalPlaneInBlockOrDiblock(int                                    detector
                                                                ,offline_plane_t                        plane_0
                                                                ,const numberOfPlanesInBlockDetector_t& planes_in_block_or_diblock
                                                                ,const BlockNumerationOffset            block_enum_offset
                                                                ) const
{

  offline_plane_t planescount = 0;
  /// Add all planes before the current diblock
  /// we index from 1 on diblocks
  const uint32_t max_number_of_blocks_or_diblocks = planes_in_block_or_diblock[detector].size() - block_enum_offset;
  for(uint32_t iblock_or_diblock = block_enum_offset; iblock_or_diblock <= max_number_of_blocks_or_diblocks; iblock_or_diblock++){

      uint32_t adding = planes_in_block_or_diblock[detector][iblock_or_diblock];

      if(planescount+adding > plane_0) break;

      planescount += adding;
  }// end of loop over blocks or diblocks

  return plane_0 - planescount;
}

//------------------------------------------------------------------------------
offline_plane_t DAQChannelMapBaseOLD::getLocalPlaneInBlockOrDiblock( const int                   detector
                                                                 ,const offline_plane_t       plane_0
                                                                 ,const uint32_t              max_number_of_blocks_or_diblocks
                                                                 ,const uint**                planes_in_block_or_diblock
                                                                 ,const BlockNumerationOffset block_enum_offset
                                                                ) const{

  offline_plane_t planescount = 0;
  /// Add all planes before the current diblock
  /// we index from 1 on diblocks
  for(uint32_t iblock_or_diblock = block_enum_offset; iblock_or_diblock < max_number_of_blocks_or_diblocks; iblock_or_diblock++){

      uint32_t adding = planes_in_block_or_diblock[detector][iblock_or_diblock];

      if(planescount+adding > plane_0) break;

      planescount += adding;
  }// end of loop over blocks or diblocks

  return plane_0 - planescount;

}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapBaseOLD::detPlaneToDetView(DetPlane_TYPE detector_plane_type) const {
    if(detector_plane_type == XZ_PLANE)return X_VIEW;
    return Y_VIEW;
}

//------------------------------------------------------------------------------
DetPlane_TYPE DAQChannelMapBaseOLD::detViewToDetPlane(DetView_TYPE view_type) const {
    if(view_type == X_VIEW)return XZ_PLANE;
    return YZ_PLANE;
}

//------------------------------------------------------------------------------
DetView_TYPE DAQChannelMapBaseOLD::getOtherView(DetView_TYPE input_view) const {
    if(input_view == X_VIEW)return Y_VIEW;
    return X_VIEW;
}

//------------------------------------------------------------------------------
DetPlane_TYPE  DAQChannelMapBaseOLD::getOtherPlaneType(DetPlane_TYPE input_plane_type) const {
    if(input_plane_type == XZ_PLANE)return YZ_PLANE;
    return XZ_PLANE;
}

//------------------------------------------------------------------------------
const char* DAQChannelMapBaseOLD::errorString(DAQChanMapErrorCodes err_code) const {
  switch(err_code){
  case  ERROR_GENERAL:              return "General Error"; break;
  case  ERROR_UNKNOWN:              return "Unknown Error"; break;
  case  ERROR_UNDEFINED:            return "Undefined Error"; break;
  case  ERROR_CHAN_OUTRANGE:        return "Channel Out of Range"; break;
  case  ERROR_PLANE_OUTRANGE:       return "Plane Out of Range"; break;
  case  ERROR_BLOCK_OUTRANGE:       return "Block Out of Range"; break;
  case  ERROR_DIBLOCK_OUTRANGE:     return "DiBlock Out of Range"; break;
  case  ERROR_DCM_OUTRANGE:         return "DCM Out of Range"; break;
  case  ERROR_PORT_OUTRANGE:        return "Port Out of Range"; break;
  case  ERROR_PIXEL_OUTRANGE:       return "Pixel Out of Range"; break;
  case  ERROR_CELL_OUTRANGE:        return "Cell Out of Range"; break;
  case  ERROR_MODULE_OUTRANGE:      return "Module Out of Range"; break;
  case  ERROR_UNKNOWN_DIBLOCK:      return "Unknown Diblock"; break;
  case  ERROR_UNKNOWN_DIBLOCK_TYPE: return "Unknown Diblock Type"; break;
  case  ERROR_UNKNOWN_DCM:          return "Unknown DCM"; break;
  case  ERROR_UNKNOWN_LINK:         return "Unknown Link"; break;
  case  ERROR_UNKNOWN_VIEW:         return "Unknown Readout View"; break;
  case  ERROR_UNKNOWN_DET:          return "Unknown Detector"; break;
  case  ERROR_UNKNOWN_CELL:         return "Unknown Cell"; break;

  default:
    return "Error Code Not Found";
    break;
  }

  return "Error with Error";
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::allocateArray(uint32_t**& outputarray, const std::vector< std::vector<uint32_t> >& input_vector){

    outputarray = (unsigned int**)malloc(input_vector.size() * sizeof(unsigned int*));

    for(uint32_t i = 0; i < input_vector.size(); ++i){
            if(input_vector[i].size() > 0)
                outputarray[i] = (uint32_t*)&input_vector[i][0];
    }
    
    return true;
}

//------------------------------------------------------------------------------
bool DAQChannelMapBaseOLD::allocateSize (uint32_t*& outputarray, const std::vector< std::vector<uint32_t> >& input_vector){

    outputarray = (unsigned int*)malloc(input_vector.size() * sizeof(unsigned int));

    for(uint32_t i = 0; i < input_vector.size(); ++i){
                outputarray[i] = input_vector[i].size();
    }

    return true;

}
